﻿Imports System.IO
Imports System.IO.Compression
Imports System.IO.Packaging

Public Class ZipfileClass
    'Add a references to System.IO.Compression, System.IO.Compression.FileStream
    Dim utils As New CommonUtilities.CommonUtilities

    Public Function GetZipDirectory(ZipFullFilename As String) As List(Of String)
        Dim result As New List(Of String)
        Try
            Dim Message = "Filename: {0}, Content Type: {1}, Compression Option: {2}, Path: {3}"
        Using pkgFile As Package = Package.Open(ZipFullFilename, FileMode.Open, FileAccess.Read)
                For Each pkgPart As PackagePart In pkgFile.GetParts()
                    Dim filename As String = FilenameFromUri(pkgPart.Uri)
                    result.Add(String.Format(Message, filename, pkgPart.ContentType, pkgPart.CompressionOption, pkgPart.Uri.ToString))
                Next
            End Using
        Catch ex As Exception
            Throw New Exception("ZipViaFramework40: ZipfileClass: GetZipDirectory: Error: " & ex.Message, ex)
        End Try
        Return result
    End Function

    Public Function ExtractZipToFolder(ZipFullFilename As String, FolderPath As String) As String
        'Returns status message to display
        Try
            If Not IO.File.Exists(ZipFullFilename) Then _
                Throw New IO.FileNotFoundException(
                    "ZipViaFramework40: ZipfileClass: ExtractZipToFolder: Error: ", ZipFullFilename)

            If Not IO.Directory.Exists(FolderPath) Then _
                IO.Directory.CreateDirectory(FolderPath)

            Dim itemCount = New IO.DirectoryInfo(FolderPath).GetFiles().Count + New IO.DirectoryInfo(FolderPath).GetDirectories.Count
            If itemCount > 0 Then
                'Enable this if you are comfortable to clear the folder before extracting to it
                'Call utils.DeleteDirectoryContents(FolderPath)
            End If

            ExtractToDirectory(ZipFullFilename, FolderPath)
        Catch ex As Exception
            Throw New Exception("ZipViaFramework40: ZipfileClass: ExtractZipToFolder: Error: " & ex.Message, ex)
        End Try
        Dim message = String.Format("Extracted zip file: {0} to folder {1}", ZipFullFilename, FolderPath)
        Return message
    End Function

    Public Function CreateZipFromFolder(ZipFullFilename As String, FolderPath As String, Optimization As CompressionOption) As String
        'Returns status message to display
        Try
            'Ensure FolderPath is not a relative path
            If (FolderPath.Trim().Length < 4) OrElse (FolderPath.Substring(1, 1) <> ":") Then
                Throw New IO.DirectoryNotFoundException(
                    "ZipViaFramework40: ZipfileClass: CreateZipFromFolder: Error: Not Absolute Folder Path: " & FolderPath)
            End If

            If IO.Directory.Exists(FolderPath) = False Then _
                Throw New IO.DirectoryNotFoundException(
                    "ZipViaFramework40: ZipfileClass: CreateZipFromFolder: Error: " & FolderPath)

            If IO.File.Exists(ZipFullFilename) Then _
                IO.File.Delete(ZipFullFilename)

            'Get list of files in directory tree from FolderPath down
            'Note: Does not capture an empty folder
            Dim files() = IO.Directory.GetFiles(FolderPath, "*.*", SearchOption.AllDirectories)
            If FolderPath.EndsWith("\") Then FolderPath = FolderPath.Substring(0, FolderPath.Length - 1)

            Using Zip As Package = ZipPackage.Open(ZipFullFilename, FileMode.OpenOrCreate, FileAccess.ReadWrite)
                For Each item In files
                    Dim relFile = item.Substring(FolderPath.Length)
                    AddFileToZip(Zip, item, relFile, Optimization)
                Next
            End Using
        Catch ex As Exception
            Throw New Exception("ZipViaFramework40: ZipfileClass: ExtractZipToFolder: Error: " & ex.Message, ex)
        End Try
        Dim message = String.Format("Created zip file: {0} from folder {1}", ZipFullFilename, FolderPath)
        Return message
    End Function

    Private Sub AddFileToZip(ByVal Zip As Package, ByVal FullFilename As String, RelativeFileName As String, Optimization As CompressionOption)
        If Not File.Exists(FullFilename) Then
            Throw New FileNotFoundException("ZipViaFramework40: ZipfileClass: AddFileToZip: Error: " & FullFilename)
        End If

        'Create a URI from the filename to zip (to ensure the name is valid)
        Dim fileURI As New Uri(UriFromFilename(RelativeFileName), UriKind.Relative)

        Dim filePart As PackagePart = Zip.CreatePart(fileURI, _
                                                     Net.Mime.MediaTypeNames.Application.Zip, _
                                                     Optimization)
        'Read the file into a byte array to can be written to Zip stream
        Dim arrBuffer As Byte() = File.ReadAllBytes(FullFilename)
        filePart.GetStream().Write(arrBuffer, 0, arrBuffer.Length)
    End Sub

    Private Sub ExtractToDirectory(ByVal ZipFullFilename As String, ByVal FolderPath As String)
        Using pkgFile As Package = Package.Open(ZipFullFilename, FileMode.Open, FileAccess.Read)
            For Each pkgPart As PackagePart In pkgFile.GetParts()
                Dim filename As String = Path.Combine(FolderPath, FilenameFromUri(pkgPart.Uri))
                Call CreateDirectoryPathIfNeeded(filename)
                Using stmSource As Stream = pkgPart.GetStream(FileMode.Open, FileAccess.Read)
                    Using stmDestination As Stream = File.OpenWrite(filename)
                        Dim arrBuffer(10000) As Byte
                        Dim intRead As Integer
                        intRead = stmSource.Read(arrBuffer, 0, arrBuffer.Length)
                        While intRead > 0
                            stmDestination.Write(arrBuffer, 0, intRead)
                            intRead = stmSource.Read(arrBuffer, 0, arrBuffer.Length)
                        End While
                    End Using
                End Using
            Next
        End Using
    End Sub

    Private Shared Function FilenameFromUri(ByVal URI As Uri) As String
        Dim FileName = URI.OriginalString.Replace("/", "\")
        If FileName.StartsWith("\") AndAlso FileName.Length > 1 Then
            FileName = FileName.Substring(1)
        End If
        Return FileName
    End Function

    Private Function UriFromFilename(ByVal FileName As String) As String
        'Replace spaces with underscores
        FileName = FileName.Replace(" "c, "_"c)

        'If a : in second character, then an absolute path, else relative
        Dim relative As Boolean = True
        If (FileName.Length > 2) AndAlso (FileName.Substring(1, 1) = ":") Then relative = False

        'A Uri MUST start with a /
        If Not relative Then
            FileName = String.Concat("/", IO.Path.GetFileName(FileName))
        Else
            FileName = FileName.Replace("\", "/")
            If FileName.StartsWith("/") = False Then FileName = "/" & FileName
        End If

        Return FileName
    End Function

    Public Sub CreateDirectoryPathIfNeeded(FullFilename As String)
        Dim dirName As String = IO.Path.GetDirectoryName(FullFilename)
        If Not IO.Directory.Exists(dirName) Then
            IO.Directory.CreateDirectory(dirName)
        End If
        Return
    End Sub

    Public Sub ArticleSampleCodeListing2(ZipFullFilename As String, FolderPath As String, Optimization As CompressionOption) 'Ensure compilation on condensed code
        'Get list of files in directory tree from FolderPath (Does not capture an empty folder)
        Dim files() = IO.Directory.GetFiles(FolderPath, "*.*", SearchOption.AllDirectories)
        If FolderPath.EndsWith("\") Then FolderPath = FolderPath.Substring(0, FolderPath.Length - 1)
        Using Zip As Package = ZipPackage.Open(ZipFullFilename, FileMode.OpenOrCreate, FileAccess.ReadWrite)
            For Each item In files
                Dim relFile = item.Substring(FolderPath.Length)
                relFile = relFile.Replace(" "c, "_"c).Replace("\", "/")
                If relFile.StartsWith("/") = False Then relFile = "/" & relFile
                Dim filePart As PackagePart = Zip.CreatePart(New Uri(relFile), _
                          Net.Mime.MediaTypeNames.Application.Zip, Optimization)
                'Read the file into a byte array that can be written to Zip stream
                Dim arrBuffer As Byte() = File.ReadAllBytes(item)
                filePart.GetStream().Write(arrBuffer, 0, arrBuffer.Length)
            Next
        End Using
    End Sub

    Public Sub ArticleSampleCodeListing3(ByVal ZipFullFilename As String, ByVal FolderPath As String)
        Using pkgFile As Package = Package.Open(ZipFullFilename, FileMode.Open, FileAccess.Read)
            For Each pkgPart As PackagePart In pkgFile.GetParts()
                Dim filename As String = Path.Combine(FolderPath,
                    pkgPart.Uri.OriginalString.Replace("/", "\").Substring(1))
                Dim dirName As String = IO.Path.GetDirectoryName(filename)
                If Not IO.Directory.Exists(dirName) Then IO.Directory.CreateDirectory(dirName)
                Using stmSource As Stream = pkgPart.GetStream(FileMode.Open, FileAccess.Read)
                    Using stmDestination As Stream = File.OpenWrite(filename)
                        Dim arrBuffer(10000) As Byte
                        Dim intRead As Integer
                        intRead = stmSource.Read(arrBuffer, 0, arrBuffer.Length)
                        While intRead > 0
                            stmDestination.Write(arrBuffer, 0, intRead)
                            intRead = stmSource.Read(arrBuffer, 0, arrBuffer.Length)
                        End While
                    End Using
                End Using
            Next
        End Using
    End Sub
End Class



